@@ -249,6 +249,7 @@ poppler_page_get_transition (PopplerPage *page)
 	  POPPLER_PAGE_TRANSITION_OUTWARD;
 
   transition->duration = trans->getDuration();
+  transition->duration_real = trans->getDuration();
   transition->angle = trans->getAngle();
   transition->scale = trans->getScale();
   transition->rectangular = trans->isRectangular();
@@ -1399,7 +1400,7 @@ poppler_page_get_annot_mapping (PopplerPage *page)
         mapping->annot = _poppler_annot_movie_new (annot);
 	break;
       case Annot::typeScreen:
-        mapping->annot = _poppler_annot_screen_new (annot);
+        mapping->annot = _poppler_annot_screen_new (page->document, annot);
 	break;
       case Annot::typeLine:
         mapping->annot = _poppler_annot_line_new (annot);
