@@ -14,13 +14,14 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Takashi Iwai <tiwai@suse.de>
-// Copyright (C) 2009-2014 Thomas Freitag <Thomas.Freitag@alfa.de>
+// Copyright (C) 2009-2015 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2010 Christian Feuersnger <cfeuersaenger@googlemail.com>
 // Copyright (C) 2011 Andreas Hartmetz <ahartmetz@gmail.com>
 // Copyright (C) 2011 Andrea Canciani <ranma42@gmail.com>
 // Copyright (C) 2011 Adrian Johnson <ajohnson@redneon.com>
-// Copyright (C) 2012 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2012, 2015 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2015 William Bader <williambader@hotmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -170,9 +171,8 @@ public:
   SplashOutputDev(SplashColorMode colorModeA, int bitmapRowPadA,
 		  GBool reverseVideoA, SplashColorPtr paperColorA,
 		  GBool bitmapTopDownA = gTrue,
-		  GBool allowAntialiasA = gTrue,
 		  SplashThinLineMode thinLineMode = splashThinLineDefault,
-      GBool overprintPreviewA = globalParams->getOverprintPreview());
+		  GBool overprintPreviewA = globalParams->getOverprintPreview());
 
   // Destructor.
   virtual ~SplashOutputDev();
@@ -231,6 +231,8 @@ public:
   virtual void updateBlendMode(GfxState *state);
   virtual void updateFillOpacity(GfxState *state);
   virtual void updateStrokeOpacity(GfxState *state);
+  virtual void updatePatternOpacity(GfxState *state);
+  virtual void clearPatternOpacity(GfxState *state);
   virtual void updateFillOverprint(GfxState *state);
   virtual void updateStrokeOverprint(GfxState *state);
   virtual void updateOverprintMode(GfxState *state);
@@ -359,6 +361,9 @@ public:
   virtual void setVectorAntialias(GBool vaa);
 #endif
 
+  GBool getFontAntialias() { return fontAntialias; }
+  void setFontAntialias(GBool anti) { fontAntialias = anti; }
+
   void setFreeTypeHinting(GBool enable, GBool enableSlightHinting);
 
 protected:
@@ -380,6 +385,12 @@ private:
 			  GBool dropEmptySubpaths);
   void drawType3Glyph(GfxState *state, T3FontCache *t3Font,
 		      T3FontCacheTag *tag, Guchar *data);
+#ifdef USE_CMS
+  GBool useIccImageSrc(void *data);
+  static void iccTransform(void *data, SplashBitmap *bitmap);
+  static GBool iccImageSrc(void *data, SplashColorPtr colorLine,
+			Guchar *alphaLine);
+#endif
   static GBool imageMaskSrc(void *data, SplashColorPtr line);
   static GBool imageSrc(void *data, SplashColorPtr colorLine,
 			Guchar *alphaLine);
@@ -396,7 +407,7 @@ private:
   int bitmapRowPad;
   GBool bitmapTopDown;
   GBool bitmapUpsideDown;
-  GBool allowAntialias;
+  GBool fontAntialias;
   GBool vectorAntialias;
   GBool overprintPreview;
   GBool enableFreeTypeHinting;
